@@ -23,71 +23,42 @@ struct iterable {
      *  - func(flecs::entity e, Components& ...)
      *  - func(flecs::iter& it, size_t index, Components& ....)
      *  - func(Components& ...)
-     * 
-     * Each iterators are automatically instanced.
      */
     template <typename Func>
     void each(Func&& func) const {
-        each(nullptr, FLECS_FWD(func));
+        ecs_iter_t it = this->get_iter(nullptr);
+        ecs_iter_next_action_t next = this->next_action();
+        while (next(&it)) {
+            _::each_delegate<Func, Components...>(func).invoke(&it);
+        }
     }
 
+    /** Run iterator.
+     * The "each" iterator accepts a function that is invoked once for a query
+     * with a valid iterator. The following signature is valid:
+     *  - func(flecs::iter&)
+     */
     template <typename Func>
-    void each(flecs::world_t *world, Func&& func) const {
-        iterate<_::each_delegate>(world, FLECS_FWD(func), 
-            this->next_each_action());
-    }
-
-    template <typename Func>
-    void each(flecs::iter& it, Func&& func) const {
-        iterate<_::each_delegate>(it.world(), FLECS_FWD(func),
-            this->next_each_action());
-    }
-
-    template <typename Func>
-    void each(flecs::entity e, Func&& func) const {
-        iterate<_::each_delegate>(e.world(), FLECS_FWD(func), 
-            this->next_each_action());
+    void run(Func&& func) const {
+        ecs_iter_t it = this->get_iter(nullptr);
+        _::run_delegate<Func>(func).invoke(&it);
     }
 
     template <typename Func>
     flecs::entity find(Func&& func) const {
-        return iterate_find<_::find_delegate>(nullptr, FLECS_FWD(func), 
-            this->next_each_action());
-    }
+        ecs_iter_t it = this->get_iter(nullptr);
+        ecs_iter_next_action_t next = this->next_action();
 
-    /** Iter iterator.
-     * The "iter" iterator accepts a function that is invoked for each matching
-     * table. The following function signatures are valid:
-     *  - func(flecs::iter& it, Components* ...)
-     *  - func(Components& ...)
-     * 
-     * Iter iterators are not automatically instanced. When a result contains
-     * shared components, entities of the result will be iterated one by one.
-     * This ensures that applications can't accidentally read out of bounds by
-     * accessing a shared component as an array.
-     */
-    template <typename Func>
-    void iter(Func&& func) const { 
-        iterate<_::iter_delegate>(nullptr, FLECS_FWD(func), 
-            this->next_action());
-    }
+        flecs::entity result;
+        while (!result && next(&it)) {
+            result = _::find_delegate<Func, Components...>(func).invoke(&it);
+        }
 
-    template <typename Func>
-    void iter(flecs::world_t *world, Func&& func) const {
-        iterate<_::iter_delegate>(world, FLECS_FWD(func), 
-            this->next_action());
-    }
+        if (result) {
+            ecs_iter_fini(&it);
+        }
 
-    template <typename Func>
-    void iter(flecs::iter& it, Func&& func) const {
-        iterate<_::iter_delegate>(it.world(), FLECS_FWD(func),
-            this->next_action());
-    }
-
-    template <typename Func>
-    void iter(flecs::entity e, Func&& func) const {
-        iterate<_::iter_delegate>(e.world(), FLECS_FWD(func), 
-            this->next_action());
+        return result;
     }
 
     /** Create iterator.
@@ -95,6 +66,16 @@ struct iterable {
      */
     iter_iterable<Components...> iter(flecs::world_t *world = nullptr) const;
 
+    /** Create iterator.
+     * Create an iterator object that can be modified before iterating.
+     */
+    iter_iterable<Components...> iter(flecs::iter& iter) const;
+
+    /** Create iterator.
+     * Create an iterator object that can be modified before iterating.
+     */
+    iter_iterable<Components...> iter(flecs::entity e) const;
+
     /** Page iterator.
      * Create an iterator that limits the returned entities with offset/limit.
      * 
@@ -129,6 +110,37 @@ struct iterable {
         return this->iter().first();
     }
 
+    iter_iterable<Components...> set_var(int var_id, flecs::entity_t value) {
+        return this->iter().set_var(var_id, value);
+    }
+
+    iter_iterable<Components...> set_var(const char *name, flecs::entity_t value) {
+        return this->iter().set_var(name, value);
+    }
+
+    iter_iterable<Components...> set_var(const char *name, flecs::table_t *value) {
+        return this->iter().set_var(name, value);
+    }
+
+    iter_iterable<Components...> set_var(const char *name, ecs_table_range_t value) {
+        return this->iter().set_var(name, value);
+    }
+
+    iter_iterable<Components...> set_var(const char *name, flecs::table_range value) {
+        return this->iter().set_var(name, value);
+    }
+
+    // Limit results to tables with specified group id (grouped queries only)
+    iter_iterable<Components...> set_group(uint64_t group_id) {
+        return this->iter().set_group(group_id);
+    }
+
+    // Limit results to tables with specified group id (grouped queries only)
+    template <typename Group>
+    iter_iterable<Components...> set_group() {
+        return this->iter().template set_group<Group>();
+    }
+
     virtual ~iterable() { }
 protected:
     friend iter_iterable<Components...>;
@@ -137,36 +149,6 @@ protected:
 
     virtual ecs_iter_t get_iter(flecs::world_t *stage) const = 0;
     virtual ecs_iter_next_action_t next_action() const = 0;
-    virtual ecs_iter_next_action_t next_each_action() const = 0;
-
-    template < template<typename Func, typename ... Comps> class Delegate, typename Func, typename NextFunc, typename ... Args>
-    void iterate(flecs::world_t *stage, Func&& func, NextFunc next, Args &&... args) const {
-        ecs_iter_t it = this->get_iter(stage);
-        if (Delegate<Func, Components...>::instanced()) {
-            ECS_BIT_SET(it.flags, EcsIterIsInstanced);
-        }
-
-        while (next(&it, FLECS_FWD(args)...)) {
-            Delegate<Func, Components...>(func).invoke(&it);
-        }
-    }
-
-    template < template<typename Func, typename ... Comps> class Delegate, typename Func, typename NextFunc, typename ... Args>
-    flecs::entity iterate_find(flecs::world_t *stage, Func&& func, NextFunc next, Args &&... args) const {
-        ecs_iter_t it = this->get_iter(stage);
-        if (Delegate<Func, Components...>::instanced()) {
-            ECS_BIT_SET(it.flags, EcsIterIsInstanced);
-        }
-
-        flecs::entity result;
-        while (!result && next(&it, FLECS_FWD(args)...)) {
-            result = Delegate<Func, Components...>(func).invoke(&it);
-        }
-        if (result) {
-            ecs_iter_fini(&it);
-        }
-        return result;
-    }
 };
 
 template <typename ... Components>
@@ -174,20 +156,51 @@ struct iter_iterable final : iterable<Components...> {
     template <typename Iterable>
     iter_iterable(Iterable *it, flecs::world_t *world) 
     {
-        m_it = it->get_iter(world);
-        m_next = it->next_action();
-        m_next_each = it->next_action();
+        it_ = it->get_iter(world);
+        next_ = it->next_action();
+        next_each_ = it->next_action();
+        ecs_assert(next_ != nullptr, ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(next_each_ != nullptr, ECS_INTERNAL_ERROR, NULL);
     }
 
     iter_iterable<Components...>& set_var(int var_id, flecs::entity_t value) {
         ecs_assert(var_id != -1, ECS_INVALID_PARAMETER, 0);
-        ecs_iter_set_var(&m_it, var_id, value);
+        ecs_iter_set_var(&it_, var_id, value);
         return *this;
     }
 
-#   ifdef FLECS_RULES
-#   include "../mixins/rule/iterable.inl"
-#   endif
+    iter_iterable<Components...>& set_var(const char *name, flecs::entity_t value) {
+        ecs_query_iter_t *qit = &it_.priv_.iter.query;
+        int var_id = ecs_query_find_var(qit->query, name);
+        ecs_assert(var_id != -1, ECS_INVALID_PARAMETER, name);
+        ecs_iter_set_var(&it_, var_id, value);
+        return *this;
+    }
+
+    iter_iterable<Components...>& set_var(const char *name, flecs::table_t *value) {
+        ecs_query_iter_t *qit = &it_.priv_.iter.query;
+        int var_id = ecs_query_find_var(qit->query, name);
+        ecs_assert(var_id != -1, ECS_INVALID_PARAMETER, name);
+        ecs_iter_set_var_as_table(&it_, var_id, value);
+        return *this;
+    }
+
+    iter_iterable<Components...>& set_var(const char *name, ecs_table_range_t value) {
+        ecs_query_iter_t *qit = &it_.priv_.iter.query;
+        int var_id = ecs_query_find_var(qit->query, name);
+        ecs_assert(var_id != -1, ECS_INVALID_PARAMETER, name);
+        ecs_iter_set_var_as_range(&it_, var_id, &value);
+        return *this;
+    }
+
+    iter_iterable<Components...>& set_var(const char *name, flecs::table_range value) {
+        ecs_table_range_t range;
+        range.table = value.get_table();
+        range.offset = value.offset();
+        range.count = value.count();
+        return set_var(name, range);
+    }
+
 #   ifdef FLECS_JSON
 #   include "../mixins/json/iterable.inl"
 #   endif
@@ -195,17 +208,17 @@ struct iter_iterable final : iterable<Components...> {
     // Return total number of entities in result.
     int32_t count() {
         int32_t result = 0;
-        while (m_next_each(&m_it)) {
-            result += m_it.count;
+        while (next_each_(&it_)) {
+            result += it_.count;
         }
         return result;
     }
 
     // Returns true if iterator yields at least once result.
     bool is_true() {
-        bool result = m_next_each(&m_it);
+        bool result = next_each_(&it_);
         if (result) {
-            ecs_iter_fini(&m_it);
+            ecs_iter_fini(&it_);
         }
         return result;
     }
@@ -213,48 +226,44 @@ struct iter_iterable final : iterable<Components...> {
     // Return first matching entity.
     flecs::entity first() {
         flecs::entity result;
-        if (m_next_each(&m_it) && m_it.count) {
-            result = flecs::entity(m_it.world, m_it.entities[0]);
-            ecs_iter_fini(&m_it);
+        if (next_each_(&it_) && it_.count) {
+            result = flecs::entity(it_.world, it_.entities[0]);
+            ecs_iter_fini(&it_);
         }
         return result;
     }
 
     // Limit results to tables with specified group id (grouped queries only)
     iter_iterable<Components...>& set_group(uint64_t group_id) {
-        ecs_query_set_group(&m_it, group_id);
+        ecs_iter_set_group(&it_, group_id);
         return *this;
     }
 
     // Limit results to tables with specified group id (grouped queries only)
     template <typename Group>
     iter_iterable<Components...>& set_group() {
-        ecs_query_set_group(&m_it, _::cpp_type<Group>().id(m_it.real_world));
+        ecs_iter_set_group(&it_, _::type<Group>().id(it_.real_world));
         return *this;
     }
 
 protected:
     ecs_iter_t get_iter(flecs::world_t *world) const override {
         if (world) {
-            ecs_iter_t result = m_it;
+            ecs_iter_t result = it_;
             result.world = world;
             return result;
         }
-        return m_it;
+        return it_;
     }
 
     ecs_iter_next_action_t next_action() const override {
-        return m_next;
-    }
-
-    ecs_iter_next_action_t next_each_action() const override {
-        return m_next_each;
+        return next_;
     }
 
 private:
-    ecs_iter_t m_it;
-    ecs_iter_next_action_t m_next;
-    ecs_iter_next_action_t m_next_each;
+    ecs_iter_t it_;
+    ecs_iter_next_action_t next_;
+    ecs_iter_next_action_t next_each_;
 };
 
 template <typename ... Components>
@@ -263,33 +272,41 @@ iter_iterable<Components...> iterable<Components...>::iter(flecs::world_t *world
     return iter_iterable<Components...>(this, world);
 }
 
+template <typename ... Components>
+iter_iterable<Components...> iterable<Components...>::iter(flecs::iter& it) const
+{
+    return iter_iterable<Components...>(this, it.world());
+}
+
+template <typename ... Components>
+iter_iterable<Components...> iterable<Components...>::iter(flecs::entity e) const
+{
+    return iter_iterable<Components...>(this, e.world());
+}
+
 template <typename ... Components>
 struct page_iterable final : iterable<Components...> {
     template <typename Iterable>
     page_iterable(int32_t offset, int32_t limit, Iterable *it) 
-        : m_offset(offset)
-        , m_limit(limit)
+        : offset_(offset)
+        , limit_(limit)
     {
-        m_chain_it = it->get_iter(nullptr);
+        chain_it_ = it->get_iter(nullptr);
     }
 
 protected:
     ecs_iter_t get_iter(flecs::world_t*) const {
-        return ecs_page_iter(&m_chain_it, m_offset, m_limit);
+        return ecs_page_iter(&chain_it_, offset_, limit_);
     }
 
     ecs_iter_next_action_t next_action() const {
         return ecs_page_next;
     }
 
-    ecs_iter_next_action_t next_each_action() const {
-        return ecs_page_next;
-    }
-
 private:
-    ecs_iter_t m_chain_it;
-    int32_t m_offset;
-    int32_t m_limit;
+    ecs_iter_t chain_it_;
+    int32_t offset_;
+    int32_t limit_;
 };
 
 template <typename ... Components>
@@ -303,29 +320,25 @@ page_iterable<Components...> iterable<Components...>::page(
 template <typename ... Components>
 struct worker_iterable final : iterable<Components...> {
     worker_iterable(int32_t offset, int32_t limit, iterable<Components...> *it) 
-        : m_offset(offset)
-        , m_limit(limit)
+        : offset_(offset)
+        , limit_(limit)
     {
-        m_chain_it = it->get_iter(nullptr);
+        chain_it_ = it->get_iter(nullptr);
     }
 
 protected:
     ecs_iter_t get_iter(flecs::world_t*) const {
-        return ecs_worker_iter(&m_chain_it, m_offset, m_limit);
+        return ecs_worker_iter(&chain_it_, offset_, limit_);
     }
 
     ecs_iter_next_action_t next_action() const {
         return ecs_worker_next;
     }
 
-    ecs_iter_next_action_t next_each_action() const {
-        return ecs_worker_next;
-    }
-
 private:
-    ecs_iter_t m_chain_it;
-    int32_t m_offset;
-    int32_t m_limit;
+    ecs_iter_t chain_it_;
+    int32_t offset_;
+    int32_t limit_;
 };
 
 template <typename ... Components>
