@@ -14,11 +14,13 @@ void Doc_get_entity_name(void);
 void Doc_get_set_brief(void);
 void Doc_get_set_detail(void);
 void Doc_get_set_link(void);
+void Doc_get_set_uuid(void);
 void Doc_set_name_nullptr(void);
 void Doc_set_brief_nullptr(void);
 void Doc_set_detail_nullptr(void);
 void Doc_set_link_nullptr(void);
 void Doc_set_color_nullptr(void);
+void Doc_set_uuid_nullptr(void);
 
 // Testsuite 'Pipeline'
 void Pipeline_system_order_same_phase(void);
@@ -320,6 +322,7 @@ void Stats_get_entity_count(void);
 void Stats_get_pipeline_stats_w_task_system(void);
 void Stats_get_not_alive_entity_count(void);
 void Stats_progress_stats_systems(void);
+void Stats_progress_stats_systems_w_empty_table_flag(void);
 
 // Testsuite 'Run'
 void Run_setup(void);
@@ -556,6 +559,10 @@ bake_test_case Doc_testcases[] = {
         "get_set_link",
         Doc_get_set_link
     },
+    {
+        "get_set_uuid",
+        Doc_get_set_uuid
+    },
     {
         "set_name_nullptr",
         Doc_set_name_nullptr
@@ -575,6 +582,10 @@ bake_test_case Doc_testcases[] = {
     {
         "set_color_nullptr",
         Doc_set_color_nullptr
+    },
+    {
+        "set_uuid_nullptr",
+        Doc_set_uuid_nullptr
     }
 };
 
@@ -1711,6 +1722,10 @@ bake_test_case Stats_testcases[] = {
     {
         "progress_stats_systems",
         Stats_progress_stats_systems
+    },
+    {
+        "progress_stats_systems_w_empty_table_flag",
+        Stats_progress_stats_systems_w_empty_table_flag
     }
 };
 
@@ -2523,7 +2538,7 @@ static bake_test_suite suites[] = {
         "Doc",
         NULL,
         NULL,
-        10,
+        12,
         Doc_testcases
     },
     {
@@ -2607,7 +2622,7 @@ static bake_test_suite suites[] = {
         "Stats",
         NULL,
         NULL,
-        12,
+        13,
         Stats_testcases
     },
     {
