@@ -1738,3 +1738,47 @@ void World_run_post_frame_outside_of_frame(void) {
     int ctx = 10;
     ecs_run_post_frame(world, post_frame_action, &ctx);
 }
+
+void World_get_flags(void) {
+    ecs_world_t *world = ecs_mini();
+
+    test_assert(!(ecs_world_get_flags(world) & EcsWorldFrameInProgress));
+
+    ecs_frame_begin(world, 0);
+
+    test_assert((ecs_world_get_flags(world) & EcsWorldFrameInProgress));
+
+    ecs_frame_end(world);
+
+    test_assert(!(ecs_world_get_flags(world) & EcsWorldFrameInProgress));
+
+    ecs_fini(world);
+}
+
+void World_fini_queue_overflow(void) {
+    /* This test verifies command flushing happens in batches during
+    world fini to avoid overflowing the vector holding the
+    command queue */
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    /* create a prefab entity: */
+    ecs_entity_t prefab = ecs_new(world);
+    ecs_add_id(world, prefab, EcsPrefab);
+    ecs_add_id(world, prefab, ecs_id(Position));
+
+    /* Create a large amount of entities. A number greater than 16777216,
+     2^30 / sizeof(ecs_cmd_t) would overflow the command queue vector */
+
+    ecs_bulk_init(world, &(ecs_bulk_desc_t) {
+        .count = 17000000,
+        .ids = { ecs_isa(prefab) }
+    });
+
+    /* on world fini, all entities must be destroyed in batches. */
+    ecs_fini(world);
+    
+    test_assert(true); /* if ecs_fini did not crash we're good */
+}
\ No newline at end of file
