@@ -390,6 +390,94 @@ void Observer_yield_existing_2_terms(void) {
     test_int(count, 6);
 }
 
+
+void Observer_yield_existing_on_create_flag(void) {
+    flecs::world world;
+
+    struct Tag0 { };
+    struct Tag1 { };
+
+    auto e1 = world.entity().add<Tag0>();
+    auto e2 = world.entity().add<Tag0>();
+    auto e3 = world.entity().add<Tag0>().add<Tag1>();
+
+    int32_t count = 0;
+
+    auto o = world.observer<Tag0>()
+        .event(flecs::OnAdd)
+        .event(flecs::OnRemove)
+        .observer_flags(EcsObserverYieldOnCreate)
+        .each([&](flecs::entity e, Tag0) {
+            if (e == e1) count ++;
+            if (e == e2) count += 2;
+            if (e == e3) count += 3;
+        });
+
+    test_int(count, 6);
+
+    o.destruct();
+
+    test_int(count, 6);
+}
+
+void Observer_yield_existing_on_delete_flag(void) {
+    flecs::world world;
+
+    struct Tag0 { };
+    struct Tag1 { };
+
+    auto e1 = world.entity().add<Tag0>();
+    auto e2 = world.entity().add<Tag0>();
+    auto e3 = world.entity().add<Tag0>().add<Tag1>();
+
+    int32_t count = 0;
+
+    auto o = world.observer<Tag0>()
+        .event(flecs::OnAdd)
+        .event(flecs::OnRemove)
+        .observer_flags(EcsObserverYieldOnDelete)
+        .each([&](flecs::entity e, Tag0) {
+            if (e == e1) count ++;
+            if (e == e2) count += 2;
+            if (e == e3) count += 3;
+        });
+
+    test_int(count, 0);
+
+    o.destruct();
+
+    test_int(count, 6);
+}
+
+void Observer_yield_existing_on_create_delete_flag(void) {
+    flecs::world world;
+
+    struct Tag0 { };
+    struct Tag1 { };
+
+    auto e1 = world.entity().add<Tag0>();
+    auto e2 = world.entity().add<Tag0>();
+    auto e3 = world.entity().add<Tag0>().add<Tag1>();
+
+    int32_t count = 0;
+
+    auto o = world.observer<Tag0>()
+        .event(flecs::OnAdd)
+        .event(flecs::OnRemove)
+        .observer_flags(EcsObserverYieldOnCreate|EcsObserverYieldOnDelete)
+        .each([&](flecs::entity e, Tag0) {
+            if (e == e1) count ++;
+            if (e == e2) count += 2;
+            if (e == e3) count += 3;
+        });
+
+    test_int(count, 6);
+
+    o.destruct();
+
+    test_int(count, 12);
+}
+
 void Observer_default_ctor(void) {
     flecs::world world;
 
@@ -1282,3 +1370,107 @@ void Observer_on_remove_inherited(void) {
     test_int(count, 1);
     test_assert(i == matched);
 }
+
+void Observer_on_set_after_remove_override(void) {
+    flecs::world ecs;
+
+    ecs.component<Position>().add(flecs::OnInstantiate, flecs::Inherit);
+
+    auto base = ecs.prefab()
+        .set(Position{1, 2});
+
+    auto e1 = ecs.entity().is_a(base)
+        .set(Position{10, 20});
+
+    int count = 0;
+
+    ecs.observer<Position>()
+        .event(flecs::OnSet)
+        .each([&](flecs::iter& it, size_t row, Position& p) {
+            test_assert(it.entity(row) == e1);
+            test_assert(it.src(0) == base);
+            test_int(p.x, 1);
+            test_int(p.y, 2);
+            count ++;
+        });
+
+    e1.remove<Position>();
+
+    test_int(count, 1);
+}
+
+void Observer_on_set_after_remove_override_create_observer_before(void) {
+    flecs::world ecs;
+
+    ecs.component<Position>().add(flecs::OnInstantiate, flecs::Inherit);
+
+    int count = 0;
+
+    flecs::entity base = ecs.prefab();
+    flecs::entity e1 = ecs.entity();
+
+    ecs.observer<Position>()
+        .event(flecs::OnSet)
+        .each([&](flecs::iter& it, size_t row, Position& p) {
+            test_assert(it.entity(row) == e1);
+            test_assert(it.src(0) == base);
+            count ++;
+        });
+
+    base.set(Position{1, 2});
+    e1.add<Position>().is_a(base);
+
+    test_int(count, 0);
+
+    e1.remove<Position>();
+
+    test_int(count, 1);
+}
+
+void Observer_on_set_w_override_after_delete(void) {
+    flecs::world ecs;
+
+    ecs.component<Position>().add(flecs::OnInstantiate, flecs::Inherit);
+
+    auto base = ecs.prefab()
+        .set(Position{1, 2});
+
+    auto e1 = ecs.entity().is_a(base)
+        .set(Position{10, 20});
+
+    int count = 0;
+
+    ecs.observer<Position>()
+        .event(flecs::OnSet)
+        .each([&](flecs::iter& it, size_t row, Position& p) {
+            count ++;
+        });
+
+    e1.destruct();
+
+    test_int(count, 0);
+}
+
+void Observer_on_set_w_override_after_clear(void) {
+    flecs::world ecs;
+
+    ecs.component<Position>().add(flecs::OnInstantiate, flecs::Inherit);
+
+    auto base = ecs.prefab()
+        .set(Position{1, 2});
+
+    auto e1 = ecs.entity().is_a(base)
+        .set(Position{10, 20});
+
+    int count = 0;
+
+    ecs.observer<Position>()
+        .event(flecs::OnSet)
+        .each([&](flecs::iter& it, size_t row, Position& p) {
+            count ++;
+        });
+
+    e1.clear();
+
+    test_int(count, 0);
+}
