@@ -3,7 +3,7 @@
 int main(int argc, char *argv[]) {
     ecs_world_t *world = ecs_init_w_args(argc, argv);
 
-    ecs_set(world, 0, EcsRest, {0});
+    ecs_insert(world, ecs_value(EcsRest, {0}));
 
     return ecs_fini(world);
 }
