@@ -50,8 +50,6 @@ void ChangeDetection_query_changed_after_delete(void) {
     test_bool(true, ecs_query_changed(q));
 
     ecs_iter_t it = ecs_query_iter(world, q);
-    test_bool(true, ecs_query_changed(q));
-
     while (ecs_query_next(&it)) { }
     test_bool(false, ecs_query_changed(q));
 
@@ -59,7 +57,6 @@ void ChangeDetection_query_changed_after_delete(void) {
     test_bool(true, ecs_query_changed(q));
 
     it = ecs_query_iter(world, q);
-    test_bool(true, ecs_query_changed(q));
     while (ecs_query_next(&it)) { }
     test_bool(false, ecs_query_changed(q));
 
@@ -83,7 +80,6 @@ void ChangeDetection_query_changed_after_add(void) {
     test_bool(true, ecs_query_changed(q));
 
     ecs_iter_t it = ecs_query_iter(world, q);
-    test_bool(true, ecs_query_changed(q));
     while (ecs_query_next(&it)) { }
     test_bool(false, ecs_query_changed(q));
 
@@ -91,7 +87,6 @@ void ChangeDetection_query_changed_after_add(void) {
     test_bool(true, ecs_query_changed(q));
 
     it = ecs_query_iter(world, q);
-    test_bool(true, ecs_query_changed(q));
     while (ecs_query_next(&it)) { }
 
     test_bool(false, ecs_query_changed(q));
@@ -125,7 +120,6 @@ void ChangeDetection_query_changed_after_remove(void) {
     test_bool(true, ecs_query_changed(q));
 
     it = ecs_query_iter(world, q);
-    test_bool(true, ecs_query_changed(q));
     while (ecs_query_next(&it)) { }
     test_bool(false, ecs_query_changed(q));
 
@@ -243,6 +237,55 @@ void ChangeDetection_query_change_after_out_system(void) {
     test_int(sys_invoked, 1);
 }
 
+void ChangeDetection_query_change_after_out_query_no_data_flag(void) {
+    ecs_world_t *world = ecs_init();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_new_w(world, Position);
+
+    ecs_query_t *q_write = ecs_query(world, {
+        .expr = "[inout] Position",
+        .cache_kind = EcsQueryCacheAuto
+    });
+    test_assert(q_write != NULL);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "[in] Position",
+        .cache_kind = EcsQueryCacheAuto
+    });
+    test_assert(q != NULL);
+    test_bool(true, ecs_query_changed(q));
+    test_bool(true, ecs_query_changed(q));
+
+    {
+        ecs_iter_t it = ecs_query_iter(world, q);
+        test_bool(true, ecs_query_changed(q));
+        while (ecs_query_next(&it)) { }
+        test_bool(false, ecs_query_changed(q));
+    }
+
+    {
+        ecs_iter_t it = ecs_query_iter(world, q_write);
+        it.flags |= EcsIterNoData;
+        while (ecs_query_next(&it)) { }
+        test_bool(false, ecs_query_changed(q));
+    }
+
+    {
+        ecs_iter_t it = ecs_query_iter(world, q);
+        test_bool(false, ecs_query_changed(q));
+        while (ecs_query_next(&it)) {
+            test_bool(false, ecs_iter_changed(&it));
+        }
+        test_bool(false, ecs_query_changed(q));
+    }
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
 void ChangeDetection_query_change_after_in_system(void) {
     ecs_world_t *world = ecs_init();
 
@@ -649,94 +692,6 @@ void ChangeDetection_query_change_prefab_term_w_tag(void) {
     ecs_fini(world);
 }
 
-void ChangeDetection_query_change_skip_non_instanced(void) {
-    ecs_world_t *world = ecs_mini();
-
-    ECS_COMPONENT(world, Position);
-    ecs_add_pair(world, ecs_id(Position), EcsOnInstantiate, EcsInherit);
-    ECS_COMPONENT(world, Velocity);
-    ecs_add_pair(world, ecs_id(Velocity), EcsOnInstantiate, EcsInherit);
-    ECS_ENTITY(world, Tag, (OnInstantiate, Inherit));
-    
-    ecs_entity_t base = ecs_new_w(world, Position);
-    ecs_entity_t e1 = ecs_new_w_pair(world, EcsIsA, base);
-    ecs_entity_t e2 = ecs_new_w_pair(world, EcsIsA, base);
-    ecs_entity_t e3 = ecs_new_w_pair(world, EcsIsA, base);
-    ecs_entity_t e4 = ecs_new_w_pair(world, EcsIsA, base);
-
-    ecs_add(world, e1, Velocity);
-    ecs_add(world, e2, Velocity);
-    ecs_add(world, e3, Velocity);
-    ecs_add(world, e4, Velocity);
-
-    ecs_add(world, e3, Tag);
-    ecs_add(world, e4, Tag);
-
-    ecs_query_t *q = ecs_query(world, {
-        .expr = "Position(up IsA), [out] Velocity",
-        .cache_kind = EcsQueryCacheAuto
-    });
-    test_assert(q != NULL);
-
-    ecs_query_t *q_r = ecs_query(world, {
-        .expr = "Position(up IsA), [in] Velocity",
-        .cache_kind = EcsQueryCacheAuto
-    });
-    test_assert(q_r != NULL);
-
-    test_assert(ecs_query_changed(q_r) == true);
-    ecs_iter_t it = ecs_query_iter(world, q_r);
-    while (ecs_query_next(&it)) { }
-    test_assert(ecs_query_changed(q_r) == false);
-
-    it = ecs_query_iter(world, q);
-    int32_t count = 0;
-
-    while (ecs_query_next(&it)) {
-        test_int(it.count, 1);
-
-        if (it.entities[0] == e1) {
-            ecs_iter_skip(&it); /* Skip one entity in table 1 */
-        }
-        if (it.entities[0] == e3 || it.entities[0] == e4) {
-            ecs_iter_skip(&it); /* Skip both entities in table 1 */
-        }
-
-        count ++;
-    }
-
-    test_int(count, 4);
-
-    test_bool(ecs_query_changed(q_r), true);
-    
-    it = ecs_query_iter(world, q_r);
-    count = 0;
-
-    while (ecs_query_next(&it)) {
-        test_int(it.count, 1);
-
-        if (it.entities[0] == e1 || it.entities[0] == e2) {
-            /* Table changed, not all entities were skipped */
-            test_bool( ecs_iter_changed(&it), true );
-        }
-        if (it.entities[0] == e3 || it.entities[0] == e4) {
-            /* Table did not change, all entities were skipped */
-            test_bool( ecs_iter_changed(&it), false );
-        }
-
-        count ++;
-    }
-
-    test_int(count, 4);
-
-    test_bool(ecs_query_changed(q_r), false);
-
-    ecs_query_fini(q);
-    ecs_query_fini(q_r);
-
-    ecs_fini(world);
-}
-
 void ChangeDetection_query_changed_w_or(void) {
     ecs_world_t *world = ecs_mini();
 
